package com.spark.kafka.mongo.util;


import com.mongodb.spark.MongoSpark;
import com.spark.kafka.mongo.entity.Product;
import org.apache.spark.sql.*;

import org.apache.spark.sql.types.StructType;
import org.springframework.stereotype.Service;


@Service
public class SparkMongoConsumer {
    public static final String MONGODB_DOCUMENT_COLLECTION = "mongodb://127.0.0.1/Productsdb.products";
    private static final String KafkaBrokerEndpoint = "localhost:9092";
    private static String KafkaTopic = "products";

    public String getProductDataAndSave() {
        SparkSession sparkSession = SparkSession.builder()
                .master("local[*]")
                .appName("MongoSparkConnector")
                .config("spark.mongodb.input.uri", MONGODB_DOCUMENT_COLLECTION)
                .config("spark.mongodb.output.uri", MONGODB_DOCUMENT_COLLECTION)
                .getOrCreate();
        StructType structType = Encoders.bean(Product.class).schema();
        Dataset<Row> ds = sparkSession.read().format("kafka")
                .option("kafka.bootstrap.servers", KafkaBrokerEndpoint)
                .option("subscribe", KafkaTopic)
                .option("header", "true")
                .option("startingOffsets", "earliest")
                .load();
        Dataset<Row> rowDataset = ds
                .selectExpr("CAST(value as String) as message")
                .select(functions.from_json(functions.col("message"), structType).as("productData"))
                .select("productData.*");
        rowDataset.show();
        Dataset<Row> filteredData = rowDataset.filter(functions.col("deliveryStatus").equalTo("Shipped"));
        Dataset<Product> productDataset = filteredData.as(Encoders.bean(Product.class));
        productDataset.show();
        MongoSpark.save(productDataset);
        return "Data Persisted in MongoDB";
    }

}
